package compactTriangulations.check;

import arraybasedDS.ArrayBasedHalfedge;
import arraybasedDS.ArrayBasedWingededge;
import compactTriangulations.CompactTriangleMesh;
import compactTriangulations.CompactTriangleMesh_6nNew;
import compactTriangulations.bench.EvaluateNavigationalOperations;

public class CheckCorrectness {

	
	/**
	 * Check the correctness of the vertex degree computation <p>
	 * Remark: the sum of all vertex degrees must match 2e (twice the number of edges)
	 */
	public static boolean checkVertexDegree(CompactTriangleMesh ds) {
		System.out.print("Checking vertex degrees...");
		
		int n=ds.sizeOfVertices();
		int e=3*n-6; // number of edges in a planar triangulation
		int[] degrees=new int[n];
		
		int sum=0;
		for(int i=0;i<ds.sizeOfVertices();i++) {
			degrees[i]=ds.vertexDegree(i);
			sum=sum+ds.vertexDegree(i);
		}
		
		if(sum==2*e) {
			System.out.println("ok");
			return true;
		}
		else {
			System.out.println("error");
			for(int i=0;i<n;i++)
				System.out.println("v"+i+": "+degrees[i]);
			return false;
		}
	}

	/**
	 * Check the correctness of the vertex degree computation <p>
	 * Remark: the sum of all vertex degrees must match 2e (twice the number of edges)
	 */
	public static boolean checkVertexDegree(ArrayBasedHalfedge ds) {
		System.out.print("Checking vertex degrees...");
		
		int n=ds.sizeOfVertices();
		int e=3*n-6; // number of edges in a planar triangulation
		int[] degrees=new int[n];
		
		int sum=0;
		for(int i=0;i<ds.sizeOfVertices();i++) {
			degrees[i]=ds.vertexDegree(i);
			sum=sum+ds.vertexDegree(i);
		}
		
		if(sum==2*e) {
			System.out.println("ok");
			return true;
		}
		else {
			System.out.println("error");
			for(int i=0;i<n;i++)
				System.out.println("v"+i+": "+degrees[i]);
			return false;
		}
	}

	/**
	 * Check the correctness of the vertex degree computation <p>
	 * Remark: the sum of all vertex degrees must match 2e (twice the number of edges)
	 */
	public static boolean checkVertexDegree(ArrayBasedWingededge ds) {
		System.out.print("Checking vertex degrees...");
		
		int n=ds.sizeOfVertices();
		int e=3*n-6; // number of edges in a planar triangulation
		int[] degrees=new int[n];
		
		int sum=0;
		for(int i=0;i<ds.sizeOfVertices();i++) {
			degrees[i]=ds.vertexDegree(i);
			sum=sum+ds.vertexDegree(i);
		}
		
		if(sum==2*e) {
			System.out.println("ok");
			return true;
		}
		else {
			System.out.println("error");
			for(int i=0;i<n;i++)
				System.out.println("v"+i+": "+degrees[i]);
			return false;
		}
	}

	/**
	 * Check the correctness of the 'getTarget' operator <p>
	 * Remark:
	 */
	public static boolean checkTargetLeftFront(CompactTriangleMesh_6nNew ds) {
		System.out.print("Checking 'target' and 'leftFront' operators (for Compact Triangle New 6n)...");
		
		int n=ds.sizeOfVertices();
		int[] edges=EvaluateNavigationalOperations.getAllEdges(ds); // all edges of the triangulation
		
		for(int i=0;i<edges.length;i++) {
			int e=edges[i]; // the i-th edge
			
			int target=ds.getTarget(e);
			int leftFront=ds.getLeftFront(e);
			
			if(target<0 || target>n)
				throw new Error("Error: wrong target index");
			if(ds.isEdge(leftFront)==false)
				throw new Error("Error: wrong edge reference");
			
		}
		
		//throw new Error("To be completed");
		return true;
	}


}
