package compactTriangulations.test;

import java.nio.file.Files;
import java.nio.file.Paths;

import Jcg.geometry.Point_3;
import Jcg.mesh.MeshBuilder;
import Jcg.mesh.SharedVertexRepresentation;
import Jcg.polyhedron.Polyhedron_3;
import compactTriangulations.encoding.BinaryOFF;

/**
 * @author Luca Castelli Aleardi (Ecole Polytechnique, 2016)
 * 
 * Convert a standard OFF (text) file to binary encoding <p>
 * <p>
 * Remark: it works only for planar triangle meshes (without boundaries)
 * 
 * JVM input argument: -Xms2G -Xmx2G
 */
public class ConvertOFFToBinary {

	/**
	 * Read an OFF file (text or binary encoding) and build the compact data structure
	 */
	public static void main(String[] args) {
		System.out.println("\n---Convert standard OFF file to binary OFF---\n");
		
		if(args.length<1) {
			System.err.println("Error: wrong number of input parameters");
			System.err.println("One input file required: planar triangle mesh in standard OFF format");
			System.exit(0);
		}
		else if(Files.exists(Paths.get(args[0]))==false) {
			System.err.println("Error: input file \""+args[0]+"\" not found");
			System.exit(0);

		}
		if(args[0].endsWith(".off")==true || args[0].endsWith(".OFF")==true) {
			BinaryOFF.fromOFFToBinaryOFF(args[0], "output.binoff"); // the OFF file is saved in binary to file: "output.binoff"
			
			// read the the output binary file to check correctness
			System.out.println("\t---check correctness (rebuild the mesh from binary OFF)---");
			SharedVertexRepresentation m=BinaryOFF.fromBinaryOFFToSharedRepresentation("output.binoff");
	    	MeshBuilder<Point_3> load3D=new MeshBuilder<Point_3>();
	    	Polyhedron_3<Point_3> triangleMesh=load3D.createMesh(m.points, m.faceDegrees, m.faces);
	    	triangleMesh.isValid(false);
		}
		else {
			System.err.println("Error input file (wrong format): "+args[0]);
			System.err.println("One input file required: triangle mesh in OFF format (.off)");
			System.exit(0);			
		}
		
		System.out.println("\n---output file created: outputbin.binoff---\n");
	}

}
