package compactTriangulations.test;
import java.nio.file.Files;
import java.nio.file.Paths;

import compactTriangulations.CompactTriangleMesh_6nNew;
import compactTriangulations.check.CheckCorrectness;
import compactTriangulations.encoding.MeshDecoder;

/**
 * @author Luca Castelli Aleardi
 * 
 * Test the decoding algorithm: evaluate the performances of decoding the 6n compact representation. <p>
 * <p>
 * Use as input argument for the Java Virtual Machine (to avoid Stack overflows): 
 * -Xss100m
 */
public class TestDecoding {

	public static void decodeCompact(String inputFile) {
		MeshDecoder decoder;
		if(inputFile.endsWith(".binzip")==false) // the input data are stored in a text file
			decoder=new MeshDecoder(inputFile, true, false); // initialize the decoder
		else // the input data are stored in binary format
			decoder=new MeshDecoder(inputFile, true, true); // initialize the decoder
		
		CompactTriangleMesh_6nNew cdsDecoded=decoder.decode(); // decode and construct the compact representation
		
		CheckCorrectness.checkVertexDegree(cdsDecoded); // check correctness
		//System.out.println(cdsDecoded);
	}

	public static void main(String[] args) {
		System.out.println("--- Testing decoding procedure (from compressed binary format) ---");
		System.out.println("--- evaluate the performances of decoding the CDS6n compact representation ---");

		if(args.length<1) {
			System.err.println("Error: wrong number of input parameters");
			System.err.println("One input file required: triangle mesh in compressed binary format (.binzip)");
			System.exit(0);
		}
		else if(Files.exists(Paths.get(args[0]))==false) {
			System.err.println("Error: input file \""+args[0]+"\" not found");
			System.exit(0);
		}
		else if(args[0].endsWith(".binzip")==false) {
			System.err.println("Error: wrong input format (required input file .off)");
			System.exit(0);
		}
		
		int repeat=5; // number of runs to perform to evaluate runtime performances
		System.out.println("--- tests are repeated "+repeat+" times ---\n");
		
		for(int i=0;i<repeat;i++) {
			System.out.println("\n\t------ run  "+(i+1)+" ------\t");
			System.gc();
			decodeCompact(args[0]);
		}

		/*EvaluateNavigationalOperations.testVertexDegree(test.cdsNew6);*/
	}

}
