package compactTriangulations.test;
import java.nio.file.Files;
import java.nio.file.Paths;

import Jcg.mesh.MeshLoader;
import Jcg.polyhedron.Halfedge;
import Jcg.polyhedron.Polyhedron_3;
import Jcg.schnyderwoods.PlanarTriSchnyderWood;

import compactTriangulations.CompactTriangleMesh_6nNew;
import compactTriangulations.encoding.BinaryOFF;
import compactTriangulations.encoding.MeshEncoder;

/**
 * @author Luca Castelli Aleardi
 * 
 * Test the encoding procedure: this program takes as input a planar triangle mesh with 'n' vertices (OFF format). <p>
 * The input mesh is compressed using the encoding scheme is based on Schnyder woods, and has length 4n-6.
 * The program save the compressed representation to 2 files: encoded as text and binary files respectively. <p>
 * <p>
 * Use as input argument for the JVM: -Xms2G -Xmx2G
 */
public class TestEncoding {
	
	public static void encode(String filename) {
		// Polyhedron_3 mesh=MeshLoader.getSurfaceMesh(filename); // input triangle mesh (half-edge representation)
		Polyhedron_3 mesh=BinaryOFF.fromBinaryOFFToPolyhedron(filename);
		
		int rootEdge=1; // choice of the root edge
		PlanarTriSchnyderWood cut=new PlanarTriSchnyderWood(mesh, ((Halfedge) mesh.halfedges.get(rootEdge)).getOpposite()); // choice of the root face
		cut.performTraversal(); // compute the Schnyder orientation

		MeshEncoder encoder=new MeshEncoder(cut);
		encoder.encode();
		encoder.writeToBinaryFile("triangulation.binzip"); // write the compressed triangulation to a binary file
		
		encoder.encoding.check(); // check correctness
		//encoder.writeToFile("triangulation.txt"); // write the compressed triangulation to a text file
	}
	
	public static void main(String[] args) {
		System.out.println("\nCompressing triangulations: the triangulation is compressed and saved in text and binary formats");
		if(args.length!=1) {
			System.err.println("Error: argument missing (required input mesh in binary OFF format)");
			System.exit(0);
		}
		else if(Files.exists(Paths.get(args[0]))==false) {
			System.err.println("Error: input file \""+args[0]+"\" not found");
			System.exit(0);
		}
		if(args[0].endsWith(".binoff")==false) {
			System.err.println("Error: wrong input format (required input file .off)");
			System.exit(0);
		}

		encode(args[0]);
		System.out.println("--- encoding done ---");
	}

}
